/*
 * Copyright 2014-2018 ISP RAS (http://www.ispras.ru)
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.storage.internal;

import ru.ispras.fortress.data.Data;
import ru.ispras.fortress.expression.ExprUtils;
import ru.ispras.fortress.expression.Node;
import ru.ispras.fortress.expression.NodeValue;
import ru.ispras.fortress.expression.NodeVariable;
import ru.ispras.testbase.TestData;
import ru.ispras.testbase.knowledge.iterator.Iterator;

import java.lang.Iterable;
import java.util.HashMap;
import java.util.Map;
import java.util.NoSuchElementException;

class RawDataProvider implements Iterator<TestData> {
  private final Iterable<Map<String, String>> dataset;
  private final Map<String, Node> image;

  private java.util.Iterator<Map<String, String>> source;
  private TestData value;

  RawDataProvider(final Iterable<Map<String, String>> dataset, final Map<String, Node> bindings) {
    this.dataset = dataset;
    this.image = bindings;

    init();
  }

  @Override
  public void init() {
    source = dataset.iterator();
    value = null;

    next();
  }

  @Override
  public boolean hasValue() {
    return value != null;
  }

  @Override
  public TestData value() {
    if (null == value) {
      throw new NoSuchElementException();
    }
    return value;
  }

  @Override
  public void next() {
    if (null == source || !source.hasNext()) {
      value = null;
      return;
    }

    final Map<String, String> dataMap = source.next();
    final Map<String, Object> bindings = new HashMap<String, Object>(image);

    for (final String key : bindings.keySet()) {
      final Node node = (Node) bindings.get(key);

      if (!ExprUtils.isVariable(node)) {
        continue;
      }

      final NodeVariable var = (NodeVariable) node;
      final String stored = dataMap.get(var.getName());
      final Data data = var.getDataType().valueOf(stored, 10);

      bindings.put(key, new NodeValue(data));
    }

    value = new TestData(bindings);
  }

  @Override
  public void stop() {
    source = null;
  }

  @Override
  public Iterator<TestData> clone() {
    throw new UnsupportedOperationException();
  }
}
