/*
 * Copyright 2017 ISP RAS (http://www.ispras.ru)
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except
 * in compliance with the License. You may obtain a copy of the License at
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software distributed under the License
 * is distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing permissions and limitations under
 * the License.
 */

package ru.ispras.testbase.knowledge.iterator;

import org.junit.Assert;
import org.junit.Test;

import ru.ispras.fortress.data.types.bitvector.BitVector;

/**
 * Unit tests for {@link BoundaryValueIterator}.
 * 
 * @author <a href="mailto:protsenko@ispras.ru">Alexander Protsenko</a>
 */
public class BoundaryValueIteratorTestCase {

  private void runTest(final int size) {
    CollectionIterator<BitVector> iterator = BoundaryValueIterator.getBoundaryIterator(size);

    System.out.println("Boundary values for " + size + " size BitVector:");
    for (iterator.init(); iterator.hasValue(); iterator.next()) {
      BitVector value = iterator.value();
      System.out.println(value + "(" + Long.toHexString(value.longValue()) + ")");
    }
    System.out.println("");

    Assert.assertTrue(
        String.format("Failed: Boundary values for %x size BitVector not generated.", size),
        iterator.size() > 0);
  }

  @Test
  public void runTest1() {
    runTest(1);
  }

  @Test
  public void runTest5() {
    runTest(5);
  }

  @Test
  public void runTest11() {
    runTest(11);
  }

  @Test
  public void runTest16() {
    runTest(16);
  }

  @Test
  public void runTest32() {
    runTest(32);
  }

  @Test
  public void runTest111() {
    runTest(111);
  }
}
